# WARNING - Generated by {fusen} from dev/flat_full.Rmd: do not edit by hand


#' Calculate average values by group
#'
#' @param data The input data frame
#' @param column The name of the target variable
#' @param group_by The names of the grouping variables
#' @importFrom data.table setDT tstrsplit
#' @importFrom rlang :=
#' @return A data frame with average values calculated by group
#' @export
averageByGroup <- function(data, column, group_by) {
  group_values <- split(data[[column]], data[group_by])
  mean_values <- sapply(group_values, mean)
  result <- data.frame(Group = names(mean_values), logQij_mean = mean_values)
  data.table::setDT(result)[, {{ group_by }} := data.table::tstrsplit(Group, "[.]")]
  result <- subset(as.data.frame(result), select = -Group)
  return(result)
}

#' Subset Fixed Effect Inferred Terms
#'
#' This function subsets the tidy TMB object to extract the fixed effect inferred terms
#' along with their categorization into interaction and non-interaction terms.
#'
#' @param tidy_tmb The tidy TMB object containing the inferred terms.
#'
#' @return A list with two elements:
#' \describe{
#'   \item{fixed_term}{A list with two components - \code{nonInteraction} and \code{interaction},
#'   containing the names of the fixed effect inferred terms categorized as non-interaction and interaction terms, respectively.}
#'   \item{data}{A data frame containing the subset of tidy_tmb that contains the fixed effect inferred terms.}
#' }
#' @export
#' @examples
#' input_var_list <- init_variable()
#' mock_data <- mock_rnaseq(input_var_list, 10, 2, 2)
#' getData2computeActualFixEffect(mock_data$groundTruth$effect)
#' data2fit = prepareData2fit(countMatrix = mock_data$counts, metadata =  mock_data$metadata )
#' #-- fit data
#' resFit <- fitModelParallel(formula = kij ~ myVariable   ,
#'                            data = data2fit, group_by = "geneID",
#'                            family = glmmTMB::nbinom2(link = "log"), n.cores = 1) 
#' tidy_tmb <- tidy_tmb(resFit)
#' subsetFixEffectInferred(tidy_tmb)
subsetFixEffectInferred <- function(tidy_tmb){
  fixed_tidy <- tidy_tmb[tidy_tmb$effect == "fixed",]
  l_term <- unique(fixed_tidy$term)
  l_term <- l_term[!l_term %in% c("(Intercept)", NA)]
  index_interaction <- grepl(x = l_term, ":")
  l_term_nonInteraction <- l_term[!index_interaction]
  l_term_interaction <- l_term[index_interaction]
  l_term2ret <- list(nonInteraction = l_term_nonInteraction, interaction = l_term_interaction )
  return(list(fixed_term = l_term2ret, data = fixed_tidy))
}


#' Get data for calculating actual values
#'
#' @param groundTruth The ground truth data frame
#' @return A list containing required data for calculating actual values
#' @export
#' @examples
#' input_var_list <- init_variable()
#' mock_data <- mock_rnaseq(input_var_list, 10, 2, 2)
#' getData2computeActualFixEffect(mock_data$groundTruth$effect)
getData2computeActualFixEffect <- function(groundTruth){
  col_names <- colnames(groundTruth)
  categorical_vars <- col_names[grepl(col_names, pattern = "label_")]
  average_gt <- averageByGroup(groundTruth, "log_qij_scaled", c("geneID", categorical_vars))
  average_gt <- convert2Factor(data = average_gt, columns = categorical_vars )
  return(list(categorical_vars = categorical_vars, data = average_gt))
}


#' Get the intercept dataframe
#'
#' @param fixeEff_dataActual The input list containing  the categorical variables and the data 
#' @return The intercept dataframe
#' @export
getActualIntercept <- function(fixeEff_dataActual) {
  ## -- split list
  data<- fixeEff_dataActual$data
  categorical_vars <- fixeEff_dataActual$categorical_vars

  if (length(categorical_vars) == 1){
    l_labels <- list()
    l_labels[[categorical_vars]] <- levels(data[, categorical_vars])

  } else l_labels <- lapply(data[, categorical_vars], levels)
  index_ref <- sapply(categorical_vars, function(var) data[[var]] == l_labels[[var]][1])
  index_ref <- rowSums(index_ref) == dim(index_ref)[2]
  df_intercept <- data[index_ref, ]
  df_intercept$term <- "(Intercept)"
  colnames(df_intercept)[colnames(df_intercept) == "logQij_mean"] <- "actual"
  df_intercept$description <- "(Intercept)"

  index2keep <- !colnames(df_intercept) %in% categorical_vars
  df_intercept <- df_intercept[,index2keep]

  return(df_intercept)
}


#' Generate actual values for a given term
#'
#' @param term The term for which actual values are calculated
#' @param df_actualIntercept The intercept dataframe
#' @param dataActual The average ground truth dataframe
#' @param categorical_vars The names of the categorical variables
#' @return The data frame with actual values for the given term
#' @export
generateActualForMainFixEff <- function(term , df_actualIntercept , dataActual  , categorical_vars){
  
  computeActualValueForMainFixEff <- function(df_actualIntercept, df_term) {
        df_term$actual <- df_term$logQij_mean - df_actualIntercept$actual
        return(subset(df_term, select = -c(logQij_mean)))
  }
  
  df_term <- subsetByTermLabel(dataActual, categorical_vars , term  )
  df_term <- computeActualValueForMainFixEff(df_actualIntercept, df_term)
  df_term$description <- gsub("\\d+$", "", term)
  return(df_term)
}



#' subset data By Term Label
#'
#'
#' Get a subset of the data based on a specific term label in the categorical variables.
#'
#' @param data The data frame to subset
#' @param categorical_vars The categorical variables to consider
#' @param term_label The term label to search for
#' @return A subset of the data frame containing rows where the categorical variables match the specified term label
#' @export
#'
#' @examples
#' # Create a data frame
#' my_data <- data.frame(color = c("red", "blue", "green", "red"),
#'                       size = c("small", "medium", "large", "medium"),
#'                       shape = c("circle", "square", "triangle", "circle"))
#' my_data[] <- lapply(my_data, as.factor)
#'
#' # Get the subset for the term "medium" in the "size" variable
#' subsetByTermLabel(my_data, "size", "medium")
#' # Output: A data frame with rows where "size" is "medium"
#'
#' # Get the subset for the term "red" in the "color" variable
#' subsetByTermLabel(my_data, "color", "red")
#' # Output: A data frame with rows where "color" is "red"
subsetByTermLabel <- function(data, categorical_vars, term_label ) {
  if (length(categorical_vars) == 1) {
    l_labels <- list()
    l_labels[[categorical_vars]] <- levels(data[, categorical_vars])
  } else {
    l_labels <- lapply(data[, categorical_vars], levels)
  }

  term_variable <- findAttribute(term_label, l_labels)
  if(is.null(term_variable)) stop("term_label not in 'data'")

  index_ref <- sapply(categorical_vars, function(var) {
    if (var == term_variable) {
      data[[var]] == term_label
    } else {
      data[[var]] == l_labels[[var]][1]
    }
  })

  index_ref <- rowSums(index_ref) == dim(index_ref)[2]
  df_term <- data[index_ref, ]
  df_term$term <- term_label
  return(df_term)
}

#' Find Attribute
#'
#' Find the attribute containing the specified term in a given list.
#'
#' @param term The term to search for
#' @param list The list to search within
#' @return The attribute containing the term, or NULL if the term is not found in any attribute
#' @export
#'
#' @examples
#' # Create a list
#' my_list <- list(color = c("red", "blue", "green"),
#'                 size = c("small", "medium", "large"),
#'                 shape = c("circle", "square", "triangle"))
#'
#' # Find the attribute containing "medium"
#' findAttribute("medium", my_list)
findAttribute <- function(term, list) {
  for (attr in names(list)) {
    if (term %in% list[[attr]]) {
      return(attr)
    }
  }
  return(NULL)  # If the term is not found in any attribute
}

#' Get actual values for non-interaction terms
#'
#' @param l_term list of term to compute 
#' @param fixeEff_dataActual A list containing required data for calculating actual values
#' @param df_actualIntercept The data frame containing the actual intercept values
#' @return A data frame with actual values for non-interaction terms
#' @export
getActualMainFixEff <- function( l_term , fixeEff_dataActual , df_actualIntercept  ){
  ## -- split list
  categorical_vars <- fixeEff_dataActual$categorical_vars
  data_groundTruth <- fixeEff_dataActual$data
  ## -- iteration over term
  l_actual <- lapply(l_term,
                     function(term){
                       generateActualForMainFixEff(term, df_actualIntercept,
                                               data_groundTruth, categorical_vars)})
  df_actual <- do.call("rbind", l_actual)
  index2keep <- !colnames(df_actual) %in% categorical_vars
  df_actual <- df_actual[,index2keep]
  return(df_actual)
}

