# WARNING - Generated by {fusen} from dev/flat_full.Rmd: do not edit by hand


#' Convert count matrix to long data frame
#'
#' Converts a count matrix to a long data frame format using geneID as the identifier.
#'
#' @param countMatrix Count matrix
#' @param value_name Name for the value column
#' @param id_vars Name for the id column (default "geneID")
#' @return Long data frame
#' @importFrom reshape2 melt
#' @export
#' @examples
#' list_var <- init_variable()
#' mock_data <- mock_rnaseq(list_var, n_genes = 3, 2, 2)
#' countMatrix_2longDtf(mock_data$counts)
countMatrix_2longDtf <- function(countMatrix, value_name = "kij", id_vars = "geneID") {
  countMatrix <- as.data.frame(countMatrix)
  countMatrix$geneID <- rownames(countMatrix)
  dtf_countLong <- reshape2::melt(countMatrix, id.vars = id_vars, variable.name = "sampleID", 
                                  value.name = value_name)
  dtf_countLong$sampleID <- as.character(dtf_countLong$sampleID)
  return(dtf_countLong)
}

#' Get column name with sampleID
#'
#' Returns the column name in the metadata data frame that corresponds to the given sampleID.
#'
#' @param dtf_countsLong Long data frame of counts
#' @param metadata Metadata data frame
#' @return Column name with sampleID
#' @export
#' @examples
#' list_var <- init_variable()
#' mock_data <- mock_rnaseq(list_var, n_genes = 3, 2,2)
#' dtf_countLong <- countMatrix_2longDtf(mock_data$counts)
#' getColumnWithSampleID(dtf_countLong, mock_data$metadata)
getColumnWithSampleID <- function(dtf_countsLong, metadata) {
  example_spleID <- as.character(dtf_countsLong[1, "sampleID"])
  regex <- paste("^", as.character(dtf_countsLong[1, "sampleID"]), "$", sep = "")
  ## -- init
  name_column <- NA
  for (indice_col in 1:dim(metadata)[2]) {
    bool_column_samples <- grep(pattern = regex, metadata[, indice_col])
    if (!identical(bool_column_samples, integer(0))) {
       name_column <- colnames(metadata)[indice_col]
    } 
  }
  return(name_column)
}

#' Validates a custom expression.
#'
#' This function checks whether the provided expression is a valid R expression and can be correctly parsed.
#'
#' @param custom_expression The custom expression to validate.
#' @return TRUE if the expression is valid, FALSE otherwise.
#' @export
#' @examples
#' isValidExpression("x + 1")
#' isValidExpression("log(x +)")
isValidExpression <- function(custom_expression) {
  if (!is.character(custom_expression)) {
    message("Custom expression must be a character string.")
    return(FALSE)
  }
  
  if (length(custom_expression) != 1) {
    message("Custom expression must be a single string.")
    return(FALSE)
  }
  
  tryCatch({
    parse(text = custom_expression)
    return(TRUE)
  }, error = function(e) {
    message("Invalid custom expression:", e$message)
    return(FALSE)
  })
}

#' Applies a custom function to a count matrix.
#'
#' This function applies a custom R function to each element of the count matrix and returns the transformed matrix.
#'
#' @param count_matrix The count matrix to transform.
#' @param custom_expression The custom R expression to apply to each element of the count matrix.
#' @return The transformed count matrix.
#' @export
#' @examples
#' # Define a count matrix
#' count_matrix <- matrix(1:9, nrow = 3)
#' # Apply the expression x + 1 to each element of the count matrix
#' custom_matrix_transform(count_matrix, "x + 1")
custom_matrix_transform <- function(count_matrix, custom_expression) {
  
  isValidExpression(custom_expression)
  
  # Convertir la chaîne en expression R
  custom_expression <- parse(text = custom_expression)
  
   custom_function <- function(x) {
     eval(custom_expression, list(x = x))
   }
   transformed_matrix <- apply(count_matrix, c(1, 2), custom_function)
   return(transformed_matrix)
}



#' Prepares data for fitting.
#'
#' This function prepares the countMatrix and metadata for fitting by converting the countMatrix to a long format 
#' and joining it with metadata. Optionally, it can apply median ratio normalization and a custom transformation to the countMatrix.
#'
#' @param countMatrix Count matrix.
#' @param metadata Metadata data frame.
#' @param normalization A boolean value indicating whether to apply median ratio normalization. 
#'                     If \code{TRUE} (default), the counts matrix will be normalized using median ratio normalization. 
#'                     If \code{FALSE}, no normalization will be applied.
#' @param transform A custom R expression to apply to each element of the countMatrix. 
#'                  This expression should be provided as a character string.
#'                  For example, to apply log transformation, use `"log(x)"`. 
#'                  Note that `x` represents each element in the countMatrix. See examples for more details.
#'                  The transformation is applied before normalization (if normalization = \code{TRUE}).
#' @param response_name String referring to the target variable name that is being modeled and predicted (default: "kij").
#' @param groupID String referring to the group variable name (default: "geneID").
#' @return Data frame suitable for fitting.
#' @export
#' @examples
#' # Initialize variables and create mock RNA-Seq data
#' list_var <- init_variable()
#' mock_data <- mock_rnaseq(list_var, n_genes = 3, 2,2)
#' # Prepare data for fitting with log transformation
#' data2fit <- prepareData2fit(mock_data$counts, mock_data$metadata, transform = "log(x)")
#' # Prepare data for fitting with custom expression
#' data2fit <- prepareData2fit(mock_data$counts, mock_data$metadata, transform = "sqrt(x + 1)")
prepareData2fit <- function(countMatrix, metadata, response_name = "kij", 
                            groupID = "geneID", transform = NULL , normalization = TRUE ) {
  
  ## -- user transform
  if (!is.null(transform))
    countMatrix <- custom_matrix_transform(countMatrix, transform)
  
  ## -- median ratio normalization
  if ( isTRUE(normalization) ) {
      message("INFO: Median ratio normalization.")
      countMatrix <- medianRatioNormalization(countMatrix)
  }

  dtf_countsLong <- countMatrix_2longDtf(countMatrix, response_name)
  metadata_columnForjoining <- getColumnWithSampleID(dtf_countsLong, metadata)
  if (is.na(metadata_columnForjoining)) {
    stop("SampleIDs do not seem to correspond between countMatrix and metadata")
  }
  data2fit <- join_dtf(dtf_countsLong, metadata, k1 = "sampleID", k2 = metadata_columnForjoining)
  if (sum(is.na(data2fit[[groupID]])) > 0) {
    warning("Something went wrong. NA introduced in the geneID column. Check the coherence between countMatrix and metadata.")
  }
  return(data2fit)
}



#' Apply Median Ratio Normalization to a Counts Matrix
#'
#' This function performs median ratio normalization on a counts matrix to
#' adjust for differences in sequencing depth across samples.
#'
#' @param countsMatrix A counts matrix where rows represent genes and columns
#'                     represent samples.
#'
#' @return A normalized counts matrix after applying median ratio normalization.
#'
#' @details This function calculates the logarithm of the counts matrix,
#' computes the average log expression for each gene, and then scales each
#' sample's counts by the exponential of the difference between its average log
#' expression and the median of those averages.
#' 
#' @importFrom stats median
#'
#' @examples
#' counts <- matrix(c(100, 200, 300, 1000, 1500, 2500), ncol = 2)
#' normalized_counts <- medianRatioNormalization(counts)
#'
#' @export
medianRatioNormalization <- function(countsMatrix) {
  log_data <- log(countsMatrix)
  average_log <- rowMeans(log_data)
  
  if (all(is.infinite(average_log)))
    stop("Every gene contains at least one zero, cannot compute log geometric means")
  
  idx2keep <- average_log != "-Inf"
  average_log <- average_log[idx2keep]
  
  ratio_data <- sweep(log_data[idx2keep, ], 1, average_log, "-")
  sample_medians <- apply(ratio_data, 2, stats::median)
  
  scaling_factors <- exp(sample_medians)
  countsMatrix_normalized <- sweep(countsMatrix, 2, scaling_factors, "/")
  
  return(countsMatrix_normalized)
}


