# WARNING - Generated by {fusen} from dev/flat_full.Rmd: do not edit by hand


#' Compute Covariation from Correlation and Standard Deviations
#'
#' This function computes the covariation between two variables (A and B) given their correlation and standard deviations.
#'
#' @param corr_AB The correlation coefficient between variables A and B.
#' @param sd_A The standard deviation of variable A.
#' @param sd_B The standard deviation of variable B.
#'
#' @return The covariation between variables A and B.
#' @export
#' @examples
#' corr <- 0.7
#' sd_A <- 3
#' sd_B <- 4
#' compute_covariation(corr, sd_A, sd_B)
compute_covariation <- function(corr_AB, sd_A, sd_B) {
  cov_AB <- corr_AB * sd_A * sd_B
  return(cov_AB)
}


#' Get Standard Deviations for Variables in Correlation
#'
#' This function extracts the standard deviations for the variables involved in the correlation.
#'
#' @param list_var A list containing the variables and their attributes.
#' @param between_var A character vector containing the names of the variables involved in the correlation.
#'
#' @return A numeric vector containing the standard deviations for the variables in the correlation.
#' @export
#' @examples
#' list_var <- init_variable(name = "varA", mu = 0, sd = 5, level = 3) %>%
#'          init_variable(name = "varB", mu = 0, sd = 25, level = 3)
#' between_var <- c("varA", "varB")
#' getStandardDeviationInCorrelation(list_var, between_var)
getStandardDeviationInCorrelation <- function(list_var, between_var){
  for (var in between_var) sd_List <- getGivenAttribute(list_var, "sd")
  for (var in between_var) sd_ListFromInteraction <- getGivenAttribute(list_var$interactions, "sd")
  sd_List <- c(sd_List, sd_ListFromInteraction)
  return(unname(unlist(sd_List[between_var])))
}



#' Set Correlation between Variables
#'
#' Set the correlation between two or more variables in a simulation.
#'
#' @param list_var A list containing the variables used in the simulation, initialized using \code{\link{init_variable}}.
#' @param between_var Character vector specifying the names of the variables to set the correlation between.
#' @param corr Numeric value specifying the desired correlation between the variables.
#'
#' @return Updated \code{list_var} with the specified correlation set between the variables.
#'
#' @details The function checks if the variables specified in \code{between_var} are declared and initialized in the \code{list_var}. It also ensures that at least two variables with provided standard deviation are required to set a correlation in the simulation.
#' The specified correlation value must be within the range (-1, 1). The function computes the corresponding covariance between the variables based on the specified correlation and standard deviations.
#' The correlation information is then added to the \code{list_var} in the form of a data frame containing the correlation value and the corresponding covariance value.
#' @export
#' @examples
#' list_var <- init_variable(name = "varA", mu = 0, sd = 5, level = 3) %>%
#'             init_variable(name = "varB", mu = 0, sd = 25, level = 3)
#' list_var <- set_correlation(list_var, between_var = c("varA", "varB"), corr = 0.7)
set_correlation <- function(list_var, between_var, corr) {

  # Check if variables in between_var are declared and initialized
  bool_checkBetweenVarValidity <- function(between_var, list_var) {
    nb_varInCorrelation <- length(unique(between_var))
    stopifnot(nb_varInCorrelation > 1)
    # -- check also for interaction
    varInitialized <- c(getListVar(list_var), getListVar(list_var$interactions))
    existingVar_nb <- varInitialized  %in% between_var %>% sum()
    if (existingVar_nb != nb_varInCorrelation) {
      return(FALSE)
    } else {
      return(TRUE)
    }
  }
  
  name_correlation <- paste(between_var, collapse = ".")
  bool_valid_corr <- bool_checkBetweenVarValidity(between_var, list_var)
  if (!bool_valid_corr) {
    stop("At least one variable in between_var is not declared. Variable not initialized cannot be used in a correlation.")
  }
  
  vec_standardDev <- getStandardDeviationInCorrelation(list_var, between_var)
  if (length(vec_standardDev) < 2) {
    stop("Exactly two variables with provided standard deviation are required to set a correlation in simulation.")
  }
  # Validate the specified correlation value to be within the range [-1, 1]
  if (corr < -1 || corr > 1) {
    stop("Invalid correlation value. Correlation must be in the range [-1, 1].")
  }
  
  name_interaction <- paste(between_var, collapse = ":")
  corr <- data.frame(cor = corr, covar = compute_covariation(corr, vec_standardDev[1], vec_standardDev[2] ))
  list_var$correlations[[name_correlation]] <- corr
  return(list_var)
}


