# WARNING - Generated by {fusen} from dev/flat_full.Rmd: do not edit by hand



#' Extract Fixed Effects from a GLMMTMB Model Summary
#'
#' This function extracts fixed effects from the summary of a glmmTMB model.
#'
#' @param x A glmmTMB model object.
#' @return A dataframe containing the fixed effects and their corresponding statistics.
#' @export
#' @examples
#'
#' model <- glmmTMB::glmmTMB(Sepal.Length ~ Sepal.Width + Petal.Length, data = iris)
#' fixed_effects <- extract_fixed_effect(model)
extract_fixed_effect <- function(x){
  ss = summary(x)
  as.data.frame(ss$coefficients$cond)
  ss_reshaped <- lapply(ss$coefficients,
                        function(sub_obj){
                          if(is.null(sub_obj)) return(NULL)
                          sub_obj <- data.frame(sub_obj)
                          sub_obj$term <- removeDuplicatedWord(rownames(sub_obj))
                          rownames(sub_obj) <- NULL
                          sub_obj <- renameColumns(sub_obj)
                          sub_obj
                        }
  )

  ss_df <- do.call(rbind, ss_reshaped)
  ss_df$component <- sapply(rownames(ss_df), function(x) strsplit(x, split = "[.]")[[1]][1])
  ss_df$effect <- "fixed"
  rownames(ss_df) <- NULL
  ss_df
}



#' Extract Tidy Summary of glmmTMB Model
#'
#' This function extracts a tidy summary of the fixed and random effects from a glmmTMB model and binds them together in a data frame. Missing columns are filled with NA.
#'
#' @param glm_TMB A glmmTMB model object.
#' @param ID An identifier to be included in the output data frame.
#' @return A data frame containing a tidy summary of the fixed and random effects from the glmmTMB model.
#' @export
#' @examples
#'
#' model <- glmmTMB::glmmTMB(Sepal.Length ~ Sepal.Width + Petal.Length, data = iris)
#' tidy_summary <- getTidyGlmmTMB(glm_TMB = model, ID = "Model1")
getTidyGlmmTMB <- function(glm_TMB, ID){
  if(is.null(glm_TMB)) return(NULL)
  df1 <- extract_fixed_effect(glm_TMB)
  df1 <- build_missingColumn_with_na(df1)
  df2 <- extract_ran_pars(glm_TMB)
  df2 <- build_missingColumn_with_na(df2)
  df_2ret <- rbind(df1, df2)
  df_2ret[df_2ret == "NaN"] <- NA
  df_2ret <- df_2ret[rowSums(!is.na(df_2ret)) > 0, ] # drop rows full of NA
  df_2ret$ID <- ID
  df_2ret <- reorderColumns(df_2ret,  
                            c("ID","effect", "component", "group", "term", 
                              "estimate", "std.error", "statistic", "p.value"))
  return(df_2ret)
}



#' Extract Tidy Summary of Multiple glmmTMB Models
#'
#' This function takes a list of glmmTMB models and extracts a tidy summary of the fixed and random effects from each model. It then combines the results into a single data frame.
#'
#' @param list_tmb A list of glmmTMB model objects.
#' @return A data frame containing a tidy summary of the fixed and random effects from all glmmTMB models in the list.
#' @export
#' @examples
#' model1 <- glmmTMB::glmmTMB(Sepal.Length ~ Sepal.Width + Petal.Length + (1 | Species), data = iris)
#' model2 <- glmmTMB::glmmTMB(Petal.Length ~ Sepal.Length + Sepal.Width + (1 | Species), data = iris)
#' model_list <- list(Model1 = model1, Model2 = model2)
#' tidy_summary <- tidy_tmb(model_list)
tidy_tmb <- function(list_tmb){
    if (identical(class(list_tmb), "glmmTMB")) return(getTidyGlmmTMB(list_tmb, "glmmTMB"))
    attributes(list_tmb)$names
    l_tidyRes <- lapply(attributes(list_tmb)$names,
                 function(ID)
                   {
                      glm_TMB <- list_tmb[[ID]]
                      getTidyGlmmTMB(glm_TMB, ID)
                  }
                )
    ret <- do.call("rbind", l_tidyRes)
    return(ret) 
}
  

#' Build DataFrame with Missing Columns and NA Values
#'
#' This function takes a DataFrame and a list of column names and adds missing columns with NA values to the DataFrame.
#'
#' @param df The input DataFrame.
#' @param l_columns A character vector specifying the column names to be present in the DataFrame.
#' @return A DataFrame with missing columns added and filled with NA values.
#' @export
#' @examples
#'
#' df <- data.frame(effect = "fixed", term = "Sepal.Length", estimate = 0.7)
#' df_with_na <- build_missingColumn_with_na(df)
build_missingColumn_with_na <- function(df, l_columns = c("effect", "component", "group", 
                                                          "term", "estimate", "std.error", "statistic", "p.value")) {
  df_missing_cols <- setdiff(l_columns, colnames(df))
  # Ajouter les colonnes manquantes à df1
  if (length(df_missing_cols) > 0) {
    for (col in df_missing_cols) {
      df[[col]] <- NA
    }
  }
  return(df)
}



#' Convert Correlation Matrix to Data Frame
#'
#' This function converts a correlation matrix into a data frame containing the correlation values and their corresponding interaction names.
#'
#' @param corr_matrix A correlation matrix to be converted.
#' @return A data frame with the correlation values and corresponding interaction names.
#' @export
#' @examples
#' mat <- matrix(c(1, 0.7, 0.5, 0.7, 
#'                  1, 0.3, 0.5, 0.3, 1), 
#'                  nrow = 3, 
#'                  dimnames = list(c("A", "B", "C"), 
#'                                  c("A", "B", "C")))
#' correlation_matrix_2df(mat)
correlation_matrix_2df <- function(corr_matrix){
  vec_corr <- corr_matrix[lower.tri(corr_matrix)]
  col_names <- removeDuplicatedWord(colnames(corr_matrix))
  row_names <- removeDuplicatedWord(rownames(corr_matrix))
  interaction_names <- vector("character", length(vec_corr))
  k <- 1
  n <- nrow(corr_matrix)
  for (i in 1:(n - 1)) {
    for (j in (i + 1):n) {
      interaction_names[k] <- paste("cor__", paste(col_names[i], ".", row_names[j], sep = ""), sep ="")
      k <- k + 1
    }
  }
  names(vec_corr) <- interaction_names
  ret <- data.frame(estimate = vec_corr)
  ret$term <- rownames(ret)
  rownames(ret) <- NULL
  ret
}

#' Wrapper for Extracting Variance-Covariance Components
#'
#' This function extracts variance-covariance components from a glmmTMB model object for a specific grouping factor and returns them as a data frame.
#'
#' @param var_cor A variance-covariance object from the glmmTMB model.
#' @param elt A character indicating the type of effect, either "cond" or "zi".
#' @return A data frame containing the standard deviation and correlation components for the specified grouping factor.
#' @export
#' @examples
#' model <- glmmTMB::glmmTMB(Sepal.Length ~ Sepal.Width + Petal.Length + (1|Species), 
#'                            data = iris, family = gaussian)
#' var_cor <- summary(model)$varcor$cond
#' ran_pars_df <- wrapper_var_cor(var_cor, "Species")
wrapper_var_cor <- function(var_cor, elt){
  var_group <- attributes(var_cor)$names
  l_ret <- lapply(var_group,
         function(group)
         {
           ## -- standard dev
           std_df <- data.frame(estimate = attributes(var_cor[[group]])$stddev)
           std_df$term <- paste("sd_", removeDuplicatedWord(rownames(std_df)), sep = "")
           ## -- correlation
           corr_matrix <- attributes(var_cor[[group]])$correlation
           #no correlation 2 return 
           if (nrow(corr_matrix) <= 1) ret <-  std_df
           else {
            corr_df <- correlation_matrix_2df(corr_matrix)
            ret <- rbind(std_df, corr_df)
          }
           ret$component <- elt
           ret$effect <- "ran_pars"
           ret$group <- group
           rownames(ret) <- NULL
           return(ret)
         })
  l_ret

}


#' Extract Random Parameters from a glmmTMB Model
#'
#' This function extracts the random parameters from a glmmTMB model and returns them as a data frame.
#'
#' @param x A glmmTMB model object.
#' @return A data frame containing the random parameters and their estimates.
#' @export
#' @importFrom stats setNames
#' @examples
#' model <- glmmTMB::glmmTMB(Sepal.Length ~ Sepal.Width + Petal.Length + (1|Species), data = iris, 
#'          family = gaussian)
#' random_params <- extract_ran_pars(model)
extract_ran_pars <- function(x){
  ss <- summary(x)
  l_2parcour <- c("cond", "zi")
  l_res = lapply(stats::setNames(l_2parcour, l_2parcour),
          function(elt)
            {
              var_cor <- ss$varcor[[elt]]
              return(wrapper_var_cor(var_cor, elt))
  })

  ret <- rbind(do.call("rbind", l_res$cond),do.call("rbind", l_res$zi))
  return(ret)

}


#' Rename Columns in a Data Frame
#'
#' This function renames columns in a data frame based on specified old names and corresponding new names.
#'
#' @param df A data frame.
#' @param old_names A character vector containing the old column names to be replaced.
#' @param new_names A character vector containing the corresponding new column names.
#' @return The data frame with renamed columns.
#' @export
#' @examples
#' df <- data.frame(Estimate = c(1.5, 2.0, 3.2),
#'                  Std..Error = c(0.1, 0.3, 0.2),
#'                  z.value = c(3.75, 6.67, 4.90),
#'                  Pr...z.. = c(0.001, 0.0001, 0.002))
#'
#' renamed_df <- renameColumns(df, old_names = c("Estimate", "Std..Error", "z.value", "Pr...z.."),
#'                               new_names = c("estimate", "std.error", "statistic", "p.value"))
#'
renameColumns <- function(df, old_names  = c("Estimate","Std..Error", "z.value", "Pr...z.."), 
                           new_names = c("estimate","std.error", "statistic", "p.value")) {
  stopifnot(length(old_names) == length(new_names))

  for (i in seq_along(old_names)) {
    old_col <- old_names[i]
    new_col <- new_names[i]

    if (old_col %in% names(df)) {
      names(df)[names(df) == old_col] <- new_col
    } else {
      warning(paste("Column", old_col, "not found in the data frame. Skipping renaming."))
    }
  }

  return(df)
}

