# WARNING - Generated by {fusen} from dev/flat_full.Rmd: do not edit by hand



#' Update glmmTMB models in parallel.
#'
#' This function updates glmmTMB models in parallel using multiple cores, allowing for faster computation. 
#' It updates the models with new reference labels if specified. 
#' It can also be used to fit a new formula or to change additional parameters of glmmTMB (param : "...").
#'
#' @param formula Formula for the GLMNB model.
#' @param list_tmb List of glmmTMB objects.
#' @param reference_labels Vector of reference labels. Default is c(), selecting the first alphanumeric label as reference.
#' @param n.cores Number of cores to use for parallel processing. If NULL, the function will use all available cores.
#' @param log_file File path for the log output (default: Rtmpdir/htrfit.log).
#' @param cl_type cluster type (default "PSOCK"). "FORK" is recommended for linux.
#' @param ... Additional arguments to be passed to the glmmTMB::glmmTMB function.
#' @export
#' @return A list of updated GLMNB models.
#'
#' @examples
#' # -- Example usage: update formula
#' data(iris)
#' groups <- unique(iris$Species)
#' group_by <- "Species"
#' formula <- Sepal.Length ~ Sepal.Width + Petal.Length
#' fitted_models <- fitModelParallel(formula, iris, group_by, n.cores = 1)
#' new_formula <- Sepal.Length ~ Sepal.Width 
#' results <- updateParallel(new_formula, fitted_models, n.cores = 1)
#' # Example usage: update reference
#' # -- Load the mtcars dataset
#' data("mtcars")
#' # -- Specify categorical variables
#' mtcars$vs <- factor(mtcars$vs) ## Engine (0 = V-shaped, 1 = straight)
#' levels(mtcars$vs) <- c("V-shaped", "straight")
#' mtcars$am <- factor(mtcars$am) ## Transmission (0 = automatic, 1 = manual)
#' levels(mtcars$am) <- c("automatic", "manual")
#' # -- For each group of number of cylinders:
#' # -- Explain fuel consumption with engine shape, Gross horsepower, and transmission type 
#' list_tmb <- fitModelParallel(formula = mpg ~ hp + vs + am, 
#'                    data = mtcars, group_by = "cyl", n.cores = 1)
#' # -- Relevel transmission and engine shape variables
#' list_tmb <- updateParallel(formula = mpg ~ hp + vs + am, list_tmb,
#'                   reference_labels = c("straight", "manual"), n.cores = 1)
updateParallel <- function (formula, list_tmb, reference_labels = c(), n.cores = NULL, cl_type = "PSOCK", 
                             log_file = paste(tempdir(check = FALSE), "htrfit.log", sep = "/"), 
                             ...) {
  stopifnot(is.list(list_tmb))
  non_null_idx <- first_non_null_index(list_tmb)
  if (!is.null(non_null_idx)){
      stopifnot(is.data.frame(list_tmb[[non_null_idx]]$frame))
      isValidInput2fit(list_tmb[[non_null_idx]]$frame, formula)
      list_tmb <- relevelling_factors(list_tmb, reference_labels)
      message(paste("Log file location", log_file, sep = ": "))
      list_tmb <- parallel_update(formula, list_tmb, n.cores, log_file, cl_type, ...)
      clear_memory(except_obj = list_tmb)
  }
  return(list_tmb)
}


#' Re-levels categorical variables in a the frame of a list of glmmTMB objects.
#'
#' This function re-levels categorical variables in a list of glmmTMB objects using the specified reference labels.
#'
#' @param list_tmb List of glmmTMB objects.
#' @param categorical_vars Names of the categorical variables to be re-leveled.
#' @param ref_labels Vector of reference labels corresponding to the categorical variables.
#' @return A list of glmmTMB objects with re-leveled categorical variables.
#' @export
#'
#' @examples
#' # Example usage:
#' # -- Load the mtcars dataset
#' data("mtcars")
#' ## -- specify categorical var
#' mtcars$vs <- factor(mtcars$vs) ## Engine (0 = V-shaped, 1 = straight)
#' levels(mtcars$vs) <- c("V-shaped", "straight")
#' mtcars$am <- factor(mtcars$am) ## Transmission (0 = automatic, 1 = manual)
#' levels(mtcars$am) <- c("automatic", "manual")
#' # -- For each group of number of cylinders,
#' # -- Explain fuel consumption with engine shape, Gross horsepower, and transmission type 
#' list_tmb <- fitModelParallel(formula = mpg ~ hp + vs + am, 
#'                data = mtcars, group_by = "cyl", n.cores = 1)
#' # -- Relevel transmission and engine shape variables
#' relevel_list_tmb_frame(list_tmb, c("am", "vs"), c("manual", "straight"))
relevel_list_tmb_frame <- function(list_tmb, categorical_vars, ref_labels){
  names(ref_labels) <- categorical_vars 
  lapply(list_tmb, function( tmb_obj ) {
    if (!is.null(tmb_obj)){
      for (categorical_var in names(ref_labels)) {
        reference_label <- ref_labels[categorical_var]
        tmb_obj$frame[[categorical_var]] <- stats::relevel(tmb_obj$frame[[categorical_var]], ref = reference_label)
      }
    }
    return(tmb_obj)
  })
}


#' Detects categorical variables based on reference labels in a glmmTMB object.
#'
#' This function detects categorical variables based on reference labels in a glmmTMB object's frame.
#'
#' @param tmb_frame The data frame of a glmmTMB object.
#' @param ref_labels Vector of reference labels corresponding to categorical variables.
#' @return Names of the categorical variables detected.
#' @export
#'
#' @examples
#' data("mtcars")
#' ## -- specify categorical var
#' mtcars$vs <- factor(mtcars$vs) ## Engine (0 = V-shaped, 1 = straight)
#' levels(mtcars$vs) <- c("V-shaped", "straight")
#' mtcars$am <- factor(mtcars$am) ## Transmission (0 = automatic, 1 = manual)
#' levels(mtcars$am) <- c("automatic", "manual")
#' ## -- For each group of number of cylinder,
#' ## -- explain fuel consumption with engine shape, Gross horsepower,  and transmission type
#' list_tmb <- fitModelParallel(formula = mpg ~ hp + vs + am, 
#'                  data = mtcars, group_by = "cyl" , n.cores = 1)
#' detect_categoricals_vars(list_tmb[["6"]]$frame, c("straight", "manual"))
detect_categoricals_vars <- function(tmb_frame, ref_labels){
  idx_col <- c()
  for (reference_label in ref_labels){
      
      catego_var_idx_col <- unique(which(tmb_frame == reference_label, arr.ind = T)[, "col"])
      
      if (length(catego_var_idx_col) > 1) {
        message_err <- paste("Label", reference_label, " detected in the metadata across different columns.\nUnable to determine the correct columns for re-leveling.\nPlease ensure that reference labels are specific to individual columns for re-leveling")
        stop(message_err)
      }
      
      if (length(catego_var_idx_col) == 0) {
      ref_label_str <- paste(ref_labels , collapse = ", ")
      message_err <- paste("Label", reference_label, "not found in metadata.")
      stop(message_err)
      }
      
  idx_col <- c(idx_col, catego_var_idx_col)
  }
    
  return(colnames(tmb_frame)[idx_col])
}

#' Relevels factors in a list of glmmTMB objects using specified reference labels.
#'
#' This function re-levels factors in a list of glmmTMB objects based on the specified reference labels.
#'
#' @param list_tmb List of glmmTMB objects.
#' @param reference_labels Vector of reference labels.
#' @return A list of glmmTMB objects with re-leveled factors.
#' @export
#'
#' @examples
#' data("mtcars")
#' ## -- specify categorical var
#' mtcars$vs <- factor(mtcars$vs) ## Engine (0 = V-shaped, 1 = straight)
#' levels(mtcars$vs) <- c("V-shaped", "straight")
#' mtcars$am <- factor(mtcars$am) ## Transmission (0 = automatic, 1 = manual)
#' levels(mtcars$am) <- c("automatic", "manual")
#' ## -- For each group of number of cylinder,
#' ## -- explain fuel consumption with engine shape, Gross horsepower,  and transmission type
#' list_tmb <- fitModelParallel(formula = mpg ~ hp + vs + am, 
#'                data = mtcars, group_by = "cyl", n.cores = 1 )
#' relevelling_factors(list_tmb , c("straight", "manual"))
relevelling_factors <- function(list_tmb, reference_labels ){
  if (length(reference_labels) > 0){
    l_categorical_vars <- detect_categoricals_vars(list_tmb[[1]]$frame, reference_labels)
    list_tmb <- relevel_list_tmb_frame(list_tmb, l_categorical_vars, reference_labels)
  }
  return(list_tmb)
}





#' Internal function to fit glmmTMB models in parallel.
#'
#' This function is used internally by \code{\link{updateParallel}} to fit glmmTMB models in parallel.
#'
#' @param formula Formula for the GLMNB model.
#' @param list_tmb List of glmmTMB objects.
#' @param n.cores Number of cores to use for parallel processing.
#' @param log_file File path for the log output (default : Rtmpdir/htrfit.log).
#' @param cl_type cluster type (defautl "PSOCK"). "FORK" is recommanded for linux.
#' @param ... Additional arguments to be passed to the glmmTMB::glmmTMB function.
#' @export
#' @return A list of updated GLMNB models.
#' @examples
#' data(iris)
#' groups <- unique(iris$Species)
#' group_by <- "Species"
#' formula <- Sepal.Length ~ Sepal.Width + Petal.Length
#' fitted_models <- fitModelParallel(formula, iris, group_by, n.cores = 1)
#' new_formula <- Sepal.Length ~ Sepal.Width 
#' results <- parallel_update(new_formula, fitted_models, n.cores = 1)
parallel_update <- function(formula, list_tmb, n.cores = NULL, 
                            log_file = paste(tempdir(check = FALSE), "htrfit.log", sep = "/"), 
                            cl_type = "PSOCK" , ...) {
  if (is.null(n.cores)) n.cores <-  max(1, parallel::detectCores(logical = FALSE) - 1)
  message(paste("CPU(s) number :", n.cores, sep = " "))
  message(paste("Cluster type :", cl_type, sep = " "))
  clust <- parallel::makeCluster(n.cores, type= cl_type, outfile = log_file)
  parallel::clusterExport(clust, c("launchUpdate", "fitUpdate", "is_fullrank", "drop_randfx"))
  updated_res <- parallel::parLapply(clust, X = list_tmb, fun = launchUpdate , formula = formula, ...)
  parallel::stopCluster(clust) ; invisible(gc(reset = T, verbose = F, full = T));
  return(updated_res)
}


#' Fit and update a GLMNB model.
#'
#' This function fits and updates a GLMNB model using the provided formula.
#' @param group group id to save in glmmTMB obj (usefull for update !)
#' @param glmm_obj A glmmTMB object to be updated.
#' @param formula Formula for the updated GLMNB model.
#' @param ... Additional arguments to be passed to the glmmTMB::glmmTMB function.
#' @export
#' @return An updated GLMNB model.
#'
#' @examples
#' data(iris)
#' groups <- unique(iris$Species)
#' group_by <- "Species"
#' formula <- Sepal.Length ~ Sepal.Width + Petal.Length
#' fitted_models <- fitModelParallel(formula, iris, group_by, n.cores = 1)
#' new_formula <- Sepal.Length ~ Sepal.Width 
#' updated_model <- fitUpdate("setosa", fitted_models[[1]], new_formula)
fitUpdate <- function(group, glmm_obj, formula , ...){
  data <- glmm_obj$frame
  is_fullrank(data, formula)
  resUpdt <- stats::update(glmm_obj, formula, ...)
  resUpdt$frame <- data
  ## save groupID => avoid error in future update
  resUpdt$groupId <- group
  ## family in ... => avoid error in future update
  additional_args <- list(...)
  familyArgs <- additional_args[['family']]
  if (!is.null(familyArgs)) resUpdt$call$family <- familyArgs
  ## control in ... => avoid error in future update
  controlArgs <- additional_args[['control']]
  if (!is.null(controlArgs)) resUpdt$call$control <- controlArgs
  return(resUpdt)
}


#' Launch the update process for a GLMNB model.
#'
#' This function launches the update process for a GLMNB model, capturing and handling warnings and errors.
#'
#' @param glmm_obj A glmmTMB object to be updated.
#' @param formula Formula for the updated GLMNB model.
#' @param ... Additional arguments to be passed to the glmmTMB::glmmTMB function.
#' @export
#' @return An updated GLMNB model or NULL if an error occurs.
#'
#' @examples
#' data(iris)
#' groups <- unique(iris$Species)
#' group_by <- "Species"
#' formula <- Sepal.Length ~ Sepal.Width + Petal.Length
#' fitted_models <- fitModelParallel(formula, iris, group_by, n.cores = 1)
#' new_formula <- Sepal.Length ~ Sepal.Width 
#' updated_model <- launchUpdate(fitted_models[[1]], new_formula)
launchUpdate <- function(glmm_obj, formula,  ...) {
  if (is.null(glmm_obj)) return(NULL)
  group <- glmm_obj$groupId
  tryCatch(
    expr = {
      withCallingHandlers(
        fitUpdate(group, glmm_obj, formula, ...),
        warning = function(w) {
          message(paste(Sys.time(), "warning for group", group ,":", conditionMessage(w)))
          invokeRestart("muffleWarning")
        })
    },
    error = function(e) {
    message(paste(Sys.time(), "error for group", group,":", conditionMessage(e)))
    return(NULL)
    }
  )
}

