# WARNING - Generated by {fusen} from dev/flat_full.Rmd: do not edit by hand


#' Wald test for hypothesis testing
#'
#' This function performs a Wald test for hypothesis testing by comparing an estimation
#' to a reference value using the provided standard error. It allows testing for
#' one-tailed alternatives: "greater" - β > reference_value, "less" - β < −reference_value,
#' or two-tailed alternative: "greaterAbs" - |β| > reference_value.
#' If the p-value obtained is greater than 1, it is set to 1 to avoid invalid p-values.
#'
#' @param estimation The estimated coefficient value.
#' @param std_error The standard error of the estimation.
#' @param reference_value The reference value for comparison (default is 0).
#' @param alternative The type of alternative hypothesis to test (default is "greaterAbs").
#' @return A list containing the test statistic and p-value.
#' @importFrom stats pnorm
#' @export
#' @examples
#' # Perform a Wald test with the default "greaterAbs" alternative
#' wald_test(estimation = 0.1, std_error = 0.02, reference_value = 0.2)
wald_test <- function(estimation, std_error, reference_value = 0, alternative = "greaterAbs") {
  if (alternative == "greater") {
    test_statistic <- (estimation - reference_value) / std_error
    p_value <- 1 - stats::pnorm(test_statistic, mean = 0, sd = 1, lower.tail = TRUE)
  } else if (alternative == "less") {
    test_statistic <- (estimation - reference_value) / std_error
    p_value <- pnorm(test_statistic, mean = 0, sd = 1, lower.tail = TRUE)
  } else if (alternative == "greaterAbs") {
    test_statistic <- (abs(estimation) - reference_value) / std_error
    p_value <- 2 * (1 - pnorm(test_statistic, mean = 0, sd = 1, lower.tail = TRUE))
  } else {
    stop("Invalid alternative type. Use 'greater', 'less', or 'greaterAbs'.")
  }

  # Set p-value to 1 if it exceeds 1
  p_value <- pmin(p_value, 1)
  return(list(statistic = test_statistic, p.value = p_value))
}




#' Perform statistical tests and return tidy results
#'
#' This function takes a list of glmmTMB objects and performs statistical tests based on the estimated coefficients and their standard errors. The results are returned in a tidy data frame format.
#'
#' @param list_tmb A list of glmmTMB objects representing the fitted models.
#' @param coeff_threshold  A non-negative value which specifies a ln(fold change) threshold. The Threshold  is used for the Wald test to determine whether the  coefficient (β) is significant or not, depending on \code{alt_hypothesis} parameter. Default is 0, ln(FC = 1).
#' @param alternative_hypothesis Alternative hypothesis for the Wald test (default is "greaterAbs").
#' Possible choice: 
#' "greater" 
#' - β > coeff_threshold, 
#' "less" 
#' - β < −coeff_threshold,
#' or two-tailed alternative: 
#' "greaterAbs" 
#' - |β| > coeff_threshold
#' @param correction_method a character string indicating the correction method to apply to p-values. Possible values are: 
#'                          "holm", "hochberg", "hommel", #' "bonferroni", "BH", "BY", "fdr", and "none".
#'
#' @return A tidy data frame containing the results of statistical tests for the estimated coefficients.
#'
#' @importFrom stats p.adjust
#' @export
#'
#' @examples
#' data(iris)
#' model_list <- fitModelParallel(formula = Sepal.Length ~ Sepal.Width + Petal.Length, 
#'                  data = iris, group_by = "Species", n.cores = 1) 
#' results_df <- tidy_results(model_list, coeff_threshold = 0.1, alternative_hypothesis = "greater")
tidy_results <- function(list_tmb, coeff_threshold = 0, alternative_hypothesis = "greaterAbs", correction_method = "BH") {
  tidy_tmb_df <- tidy_tmb(list_tmb)
  if (coeff_threshold != 0 || alternative_hypothesis != "greaterAbs") {
    waldRes <- wald_test(tidy_tmb_df$estimate, tidy_tmb_df$std.error, coeff_threshold, alternative_hypothesis)
    tidy_tmb_df$statistic <- waldRes$statistic
    tidy_tmb_df$p.value <- waldRes$p.value
  }
  tidy_tmb_df$p.adj <- stats::p.adjust(tidy_tmb_df$p.value, method = correction_method)
  return(tidy_tmb_df)
}


